package com.lamark.business.bemobi.api;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.lamark.architecture.corems.api.BaseController;
import com.lamark.architecture.corems.exception.BaseException;
import com.lamark.business.bemobi.core.service.BemobiService;
import com.lamark.business.bemobi.core.service.SignUpService;
import javax.inject.Inject;
import javax.ws.rs.*;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;

import com.lamark.core.dto.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

@Path("/business/v1/bemobi")
public class BemobiController extends BaseController {

    @Inject
    BemobiService bemobiService;
    private final Logger logger = LoggerFactory.getLogger(BemobiController.class);

    @Inject
    SignUpService signUpService;

    @POST
    @Consumes(MediaType.APPLICATION_FORM_URLENCODED)
    @Produces(MediaType.APPLICATION_JSON)
    @Path("/activation")
    public Response activation(@BeanParam BemobiRequestDto notification) throws BaseException {
        this.logger.info("[POST][START] /activation/{}  ", notification);
        return buildSuccessWrapperResponse(bemobiService.subscribe(notification));
    }

    @POST
    @Consumes(MediaType.APPLICATION_JSON)
    @Produces(MediaType.APPLICATION_JSON)
    @Path("/signup")
    public Response signUp(AppSessionDto signUpRequestDto) throws JsonProcessingException, BaseException {
        return buildSuccessWrapperResponse(signUpService.SignUpUser(signUpRequestDto));
    }


    @POST
    @Consumes(MediaType.APPLICATION_FORM_URLENCODED)
    @Path("/statusChange")
    public Response statusChange(@BeanParam StatusChangeDto statusChange) throws BaseException {
        this.logger.info("[POST][START] /statusChange/{}  ", statusChange);
        bemobiService.statusChange(statusChange);
        this.logger.info("[POST][END] /statusChange/{}  ", statusChange);
        return buildOkSuccessWrapperResponse (null);
    }


    @POST
    @Consumes(MediaType.APPLICATION_FORM_URLENCODED)
    @Path("/paymentNotification")
    public Response paymentNotification( @BeanParam PaymentNotificationDto paymentNotification) {

        this.logger.info("[POST][START] /paymentNotification/{}  ", paymentNotification);
        bemobiService.processPaymentNotification(paymentNotification);
        this.logger.info("[POST][END] /paymentNotification/{}  ", paymentNotification);
        return buildOkSuccessWrapperResponse (null);
    }

    @GET
    @Produces(MediaType.APPLICATION_JSON)
    @Path("/activationCredentials")
    public Response activationCredentials(@BeanParam ActivationCredentialsDto credentials) throws BaseException {
        this.logger.info("[POST][START] /activationCredentials/{}  ", credentials);
        return buildSuccessWrapperResponse(bemobiService.activateCredential(credentials));




    }
}