package com.lamark.business.bemobi.core.service.impl;

import com.lamark.architecture.corems.exception.BaseException;
import com.lamark.architecture.corems.exception.BaseRuntimeException;
import com.lamark.architecture.corems.exception.ExceptionHelper;
import com.lamark.business.bemobi.core.client.*;
import com.lamark.business.bemobi.core.service.BemobiService;
import com.lamark.core.dto.*;
import org.apache.commons.lang3.RandomStringUtils;
import org.eclipse.microprofile.config.inject.ConfigProperty;
import org.eclipse.microprofile.rest.client.inject.RestClient;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.inject.Inject;
import javax.inject.Singleton;
import javax.ws.rs.NotFoundException;
import javax.ws.rs.core.Response;
import java.text.MessageFormat;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.function.Predicate;
import java.util.stream.Collectors;

@Singleton
public class BemobiServiceImpl implements BemobiService {

    public static final int VOUCHER_LENGTH = 6;
    public static final String ME_7 = "ME7";
    public static final int SITE = 167;
    public static final String WEB = "WEB";

    @ConfigProperty(name = "url.auto.login")
    public String URL_AUTO_LOGIN;

    @Inject
    @RestClient
    DataCoreUserClient dataCoreUserClient;

    @Inject
    @RestClient
    DataCoreClient dataCoreClient;

    @Inject
    @RestClient
    BusinessCommandClient commandClient;

    @Inject
    @RestClient
    DataBemobiSubscriptionClient dataBemobiSubscription;

    @Inject
    @RestClient
    CoreClient coreClient;

    private final Logger logger = LoggerFactory.getLogger(BemobiServiceImpl.class);


    @Override
    public BemobiResponseDto subscribe(BemobiRequestDto notification) throws BaseException {

        String providerUniqueId = RandomStringUtils.randomAlphanumeric(VOUCHER_LENGTH).toUpperCase();


        SubscriptionBemobiDto susBemobiDto = new SubscriptionBemobiDto();
        susBemobiDto.setKeyword(ME_7);
        susBemobiDto.setSiteID(SITE);
        susBemobiDto.setProviderUniqueID(providerUniqueId);
        susBemobiDto.setOpxUserID(notification.getOpxUserId());
        susBemobiDto.setOpxSubscriptionID(notification.getOpxSubscriptionID());
        susBemobiDto.setExpireDate(getVoucherExpirationDate());

        List<LamarkConfigDto> lmConfSiteKeyword = dataCoreClient.getConfig("LM_CONF_SITE_KEYWORD");
        Predicate<LamarkConfigDto> sitePredicate = s -> s.getValue1().equalsIgnoreCase(String.valueOf(SITE));
        Predicate<LamarkConfigDto> keywordPredicate = s -> s.getValue2().equalsIgnoreCase(ME_7);
        Predicate<LamarkConfigDto> contentPackagePredicate = s -> "contentPackage".equalsIgnoreCase(s.getValue3());
        Predicate<LamarkConfigDto> priceCodePredicate = s -> "priceCode".equalsIgnoreCase(s.getValue3());
        Predicate<LamarkConfigDto> subscriptionDurationPredicate = s -> "subscriptionDuration".equalsIgnoreCase(s.getValue3());
        Predicate<LamarkConfigDto> amountPredicate = s -> "amount".equalsIgnoreCase(s.getValue3());

        String contentPackage = lmConfSiteKeyword
                .stream()
                .filter(sitePredicate)
                .filter(keywordPredicate)
                .filter(contentPackagePredicate)
                .findFirst()
                .map(s -> s.getValue4()) //todo
                .orElseThrow(() -> ExceptionHelper.buildNotFoundException("No se encontro content package configurado", ""));


        String priceCode = lmConfSiteKeyword
                .stream()
                .filter(sitePredicate)
                .filter(keywordPredicate)
                .filter(priceCodePredicate)
                .findFirst()
                .map(s -> s.getValue4())//todo
                .orElseThrow(() -> ExceptionHelper.buildNotFoundException("No se encontro price code configurado", ""));

        String subscriptionDuration = lmConfSiteKeyword
                .stream()
                .filter(sitePredicate)
                .filter(keywordPredicate)
                .filter(subscriptionDurationPredicate)
                .findFirst()
                .map(s -> s.getValue4())
                .orElseThrow(() -> ExceptionHelper.buildNotFoundException("No se encontro price code configurado", ""));//todo


        String amount = lmConfSiteKeyword
                .stream()
                .filter(sitePredicate)
                .filter(keywordPredicate)
                .filter(amountPredicate)
                .findFirst()
                .map(s -> s.getValue4())
                .orElseThrow(() -> ExceptionHelper.buildNotFoundException("No se encontro monto configurado", "")); //todo


        SubscriptionDto subscriptionDto = new SubscriptionDto();


        subscriptionDto.setSite(SITE);
        subscriptionDto.setMsisdn(providerUniqueId);
        subscriptionDto.setKeyword(ME_7);
        subscriptionDto.setContentPackage(Integer.valueOf(contentPackage));
        subscriptionDto.setPriceCode(priceCode);
        subscriptionDto.setAmount(amount);
        subscriptionDto.setChannel(WEB);
        subscriptionDto.setMakeBilling(!("2".equalsIgnoreCase(notification.getStatusCode()) | "13".equalsIgnoreCase(notification.getStatusCode())));//TODO evaluate cogito
        subscriptionDto.setSubscriptionDuration(Integer.valueOf(subscriptionDuration));
        commandClient.subscribe(subscriptionDto);

        BemobiResponseDto bemobiResponseDto = new BemobiResponseDto();
        bemobiResponseDto.setActivationLink(MessageFormat.format(URL_AUTO_LOGIN, SITE, providerUniqueId));
        return bemobiResponseDto;
    }

    @Override
    public void statusChange(StatusChangeDto statusChange) throws BaseException  {

        SubscriptionBemobiDto subscriptionBemobi=
                dataBemobiSubscription.findByUserAndSubscription(statusChange.getOpxUserId() , statusChange.getOpxSubscriptionId());
        Integer site = subscriptionBemobi.getSiteID();
        String countryCode = coreClient.getSiteById(site).getInternationalPrefix();
        String msisdn =  countryCode.concat(subscriptionBemobi.getProviderUniqueID());
        String keyword = subscriptionBemobi.getKeyword();


        List<LamarkConfigDto> config_by_site_keyword =  dataCoreClient
                .getConfig("LM_CONF_SITE_KEYWORD");

        Map<String, String> config1 = config_by_site_keyword.stream().filter((x) ->
                x.getValue1().equals(String.valueOf(site)) &&
                        x.getValue2().equals(keyword))
                .collect(Collectors.toMap((x) -> x.getValue3(), (x -> x.getValue4())));

        Integer subscriptionDays = Integer.valueOf(config1.get("subscriptionDuration")) ;
        Integer priceCode = Integer.valueOf(config1.get("priceCode")) ;
        Integer contentPackage = Integer.valueOf(config1.get("contentPackage"));
        String amount = config1.get("amount");

        SubscriptionDataDto subscription = new SubscriptionDataDto();
        subscription.setSiteId(site);
        subscription.setMsisdn(msisdn);
        subscription.setSubscriptionDuration(subscriptionDays);
        subscription.setPriceCodeId(priceCode);
        subscription.setContentPackageId(contentPackage);
        subscription.setAmount(amount);
        subscription.setMakeBilling(false);

        if (statusChange.getNotificationType().equals("activate_subscription")){
            subscription.setType(EventTypeDto.RENEW.name());
            subscription.setMakeBilling(true);
            commandClient.renew(subscription);
        }
        if(statusChange.getNotificationType().equals("suspend_subscription")){
            subscription.setType(EventTypeDto.SUSPEND.name());
            commandClient.suspend(msisdn,site,EventTypeDto.SUSPEND.name());
        }
        if(statusChange.getNotificationType().equals("cancel_subscription")||
                statusChange.getNotificationType().equals("parked_cancelled")||
                statusChange.getNotificationType().equals("inprogress_cancelled")
        ){
            String reasonCancelled = statusChange.getReason();

            if (reasonCancelled.equals("web_cancelled") ||
                    reasonCancelled.equals("sms_cancelled") ||
                    reasonCancelled.equals("payment_failure") ||
                    reasonCancelled.equals("in_progress_failed") ||
                    reasonCancelled.equals("signup_failed") ||
                    reasonCancelled.equals("unknown") ||
                    reasonCancelled.equals("ussd_cancelled") ){
                subscription.setType(EventTypeDto.CANCEL.name());
                commandClient.cancel(msisdn,site,EventTypeDto.CANCEL.name());

            }

            if (reasonCancelled.equals("admin_cancelled") ){
                subscription.setType(EventTypeDto.CANCEL_BY_AUTO.name());
                commandClient.cancel(msisdn,site,EventTypeDto.CANCEL_BY_AUTO.name());
            }

            if (reasonCancelled.equals("call_center") ) {
                subscription.setType(EventTypeDto.CANCEL_BY_CRM.name());
                commandClient.cancel(msisdn,site,EventTypeDto.CANCEL_BY_CRM.name());
            }
        }
    }

    @Override
    public void processPaymentNotification(PaymentNotificationDto paymentNotification) {

        if(paymentNotification.getNotificationType().equals("subscription_payment")){
            SubscriptionBemobiDto dto =  dataBemobiSubscription.findByUserAndSubscription(paymentNotification.getOpxUserId(), paymentNotification.getOpxSubscriptionId());
            String countryCode = coreClient.getSiteById(dto.getSiteID()).getInternationalPrefix();
            String msisdn = countryCode.concat(dto.getProviderUniqueID());
            UserDto userDto  = dataCoreUserClient.getUser(msisdn, dto.getSiteID());
            if(userDto.getSubscription().getState().equals(SSLStateDto.ACTIVE.getCode())){
                List<LamarkConfigDto> config_by_site_keyword =  dataCoreClient
                        .getConfig("LM_CONF_SITE_KEYWORD");

                Map<String, String> config1 = config_by_site_keyword.stream().filter((x) ->
                        x.getValue1().equals(String.valueOf(dto.getSiteID())) &&
                                x.getValue2().equals(dto.getKeyword()))
                        .collect(Collectors.toMap((x) -> x.getValue3(), (x -> x.getValue4())));

                Integer subscriptionDays = Integer.valueOf(config1.get("subscriptionDuration")) ;
                Integer priceCode = Integer.valueOf(config1.get("priceCode"))  ;
                Integer contentPackage = Integer.valueOf(config1.get("contentPackage"));
                String amount = config1.get("amount");

                SubscriptionDataDto subscription = new SubscriptionDataDto();
                subscription.setSiteId(dto.getSiteID());
                subscription.setMsisdn(msisdn);
                subscription.setSubscriptionDuration(subscriptionDays);
                subscription.setPriceCodeId(priceCode);
                subscription.setContentPackageId(contentPackage);
                subscription.setAmount(amount);
                subscription.setType(EventTypeDto.RENEW.name());
                subscription.setMakeBilling(true);
                commandClient.renew(subscription);
            }
        }
    }

    @Override
    public BemobiResponseDto activateCredential(ActivationCredentialsDto credentials) throws BaseException {
        SubscriptionBemobiDto subscriptionBemobi =
                dataBemobiSubscription.findByUserAndSubscription(credentials.getOpxUserId(),
                        credentials.getOpxSubscriptionId());
        subscriptionBemobi.setExpireDate(getVoucherExpirationDate());

        SubscriptionBemobiDto updated = dataBemobiSubscription.updateSubscription(credentials.getOpxUserId(),
                credentials.getOpxSubscriptionId(), subscriptionBemobi);


        BemobiResponseDto bemobiResponseDto = new BemobiResponseDto();
        bemobiResponseDto.setActivationLink(MessageFormat.format(URL_AUTO_LOGIN, SITE, updated.getProviderUniqueID()));
        bemobiResponseDto.setActivationTimeOut(String.valueOf(updated.getExpireDate().getTime()));

        return bemobiResponseDto;
    }

    private Date getVoucherExpirationDate() {
        Date in = new Date();
        LocalDateTime ldt = LocalDateTime.ofInstant(in.toInstant(), ZoneId.of("Asia/Karachi")).plusDays(1);
        return Date.from(ldt.atZone(ZoneId.of("Asia/Karachi")).toInstant());
    }
}
