package com.lamark.business.bemobi.core.service.impl;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.lamark.architecture.corems.exception.BaseException;
import com.lamark.architecture.corems.exception.ExceptionHelper;
import com.lamark.architecture.corems.util.DateUtil;
import com.lamark.business.bemobi.core.client.BemobiClient;
import com.lamark.business.bemobi.core.client.CoreClient;
import com.lamark.business.bemobi.core.client.DataBemobiSubscriptionClient;
import com.lamark.core.dto.*;
import com.lamark.business.bemobi.core.service.SignUpService;
import com.lamark.core.dto.SubscriptionBemobiDto;
import lombok.SneakyThrows;
import org.eclipse.microprofile.rest.client.inject.RestClient;

import javax.inject.Inject;
import javax.inject.Singleton;
import javax.ws.rs.BadRequestException;
import javax.ws.rs.NotFoundException;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.util.Date;
import java.util.List;

@Singleton
public class SingUpServiceImpl implements SignUpService {

    @Inject
    @RestClient
    CoreClient _coreClient;

    @Inject
    @RestClient
    BemobiClient bemobiClient;

    @Inject
    @RestClient
    DataBemobiSubscriptionClient dataBemobiSubscription;


    @Override
    public SignUpResponseDto SignUpUser(AppSessionDto signUpRequestDto) throws BaseException, JsonProcessingException {

        List<AppSessionDto> sessions = _coreClient.getSessions(signUpRequestDto);
        if(sessions.isEmpty()) {
            throw ExceptionHelper.buildGenericException("Sessions is EMpty", "");
        }

        SubscriptionBemobiDto bemobiSubscription = dataBemobiSubscription.findByProviderUniqueId(signUpRequestDto.getVoucher());
        if(bemobiSubscription == null) {
            throw ExceptionHelper.buildNotFoundException("Subscription not exits", "");
        }

        Date now = getZoneDate();

        if(getZoneDate().after(bemobiSubscription.getExpireDate())) {
            return SignUpResponseDto.builder().expired(true).build();
        }

        signUpRequestDto.buildRequestToSignUp(bemobiSubscription.getOpxSubscriptionID(), now.getTime());

        String response = bemobiClient.signUpComplete(
                signUpRequestDto.getOPXSubscriptionID(),
                signUpRequestDto.getServiceSignUpDate()
        );

        if (response == null || response.isEmpty()) {
            throw ExceptionHelper.buildGenericException("Response sign up is null", "");
        }

        ObjectMapper mapper = new ObjectMapper();
        return mapper.readValue(response, SignUpResponseDto.class);
    }


    public Date getZoneDate() {
        Date date = new Date();
        LocalDateTime ldt = LocalDateTime.ofInstant(date.toInstant(), ZoneId.of("Asia/Karachi"));
        return Date.from(ldt.atZone(ZoneId.of("Asia/Karachi")).toInstant());
    }
}
